from datetime import datetime

import psycopg2


class PostgreSQLHandler:

    def __init__(self, db_table, container, db_info):
        self.db_table = db_table
        self.container = container
        # DB 연결
        self.db = psycopg2.connect(
            host=db_info["host"],
            dbname=db_info["database"],
            user=db_info["user"],
            password=db_info["password"],
            port=db_info["port"]
        )
        self.cursor = self.db.cursor()

        # 테이블 생성
        self.create_table()

    def __del__(self):
        self.db.close()
        self.cursor.close()

    def create_table(self):
        create_table_sql = f"CREATE TABLE IF NOT EXISTS {self.db_table}(" \
                           "id           serial  primary key      not null," \
                           "container    varchar(50)              not null," \
                           "process      integer                  not null," \
                           "process_name varchar(50)              not null," \
                           "thread_name  varchar(50)              not null," \
                           "name         varchar(50)              not null," \
                           "thread       bigint                   not null," \
                           "pathname     varchar(200)             not null," \
                           "func_name    varchar(50)              not null," \
                           "lineno       integer                  not null," \
                           "levelname    varchar(50)              not null," \
                           "time         timestamp with time zone not null," \
                           "message      text                     not null," \
                           "exc_info     text                     not null," \
                           "exc_text     text                     not null," \
                           "stack_info   text                     not null," \
                           "levelno      integer                  not null" \
                           ");"
        self.cursor.execute(create_table_sql)
        self.db.commit()

    def insert_data(self, record):
        insert_sql = f"INSERT INTO {self.db_table} (" \
                     f"container, process, process_name, " \
                     f"thread, thread_name, " \
                     f"name, pathname, func_name, lineno, " \
                     f"levelno, levelname, time, " \
                     f"message, exc_info, exc_text, stack_info" \
                     f") VALUES (" \
                     f"'{self.container}', '{record.process}', '{record.process}', " \
                     f"'{record.thread}', '{record.threadName}', " \
                     f"'{record.name}', '{record.pathname}', '{record.funcName}', '{record.lineno}', " \
                     f"'{record.levelno}', '{record.levelname}', '{datetime.now()}', " \
                     f"'{record.message}', '{record.exc_info}', '{record.exc_text}', '{record.stack_info}'" \
                     f");"
        self.cursor.execute(insert_sql)
        self.db.commit()



import unittest
from logging import getLogger

from pycontainerutils.api.api_client import API_Client
from pycontainerutils.logger.setting_logger import Log_settings

Log_settings.logger_settings(
    logger_config={
        "": {
            "level": "DEBUG",
            "handlers": ["console"],
        },
        "test_logger_console": {
            "level": "INFO",
            "handlers": ["console"],
            "propagate": False,
        },
    },
    container_name="APIClientTest",
)

logger = getLogger(__name__)


class APIClientTest(unittest.TestCase):

    def setUp(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        pass

    def tearDown(self):
        """
        테스트에 사용하는 데이터 삭제
        """
        pass

    def test_connect(self):
        """
        sql문으로 간단한 crud가 동작하는지 확인
        djagno의 auth group table을 사용하여 테스트
        :return:
        """
        api = API_Client()
        url = f"http://apis.data.go.kr/1360000/NwpModelInfoService/getLdapsUnisAll?ServiceKey=5RR1UuzOEtgenAG4uusefJERqDYkVskem%2B%2FAEQez3aVQ%2BBMDc6iwlg6%2B%2BSVq3XoIwU%2BRg4GZS4r0PZJU1uBSJg%3D%3D&pageNo=1000&numOfRows=50000&dataType=JSON&leadHour=1&dataTypeCd=Temp&baseTime=202112050300"
        logger.debug(url)
        data = api.call_requests_repeat(url)
        if "response" in data:
            print(data)
        elif "OpenAPI_ServiceResponse" in data:
            print(data)
        pass



import time
from json.decoder import JSONDecodeError

import requests
import logging
import xmltodict
import json
logger = logging.getLogger(__name__)


class API_Client:
    timeout_seconds = 60
    waiting_time = 10
    waiting_count = 10
    total_count = 0

    def call_requests_repeat(self, url):
        response = None
        count = 0
        while count < self.waiting_count:
            count += 1
            self.total_count += 1
            try:    # 실질 api 요청
                response = requests.get(url, timeout=self.timeout_seconds)
                response.raise_for_status()
                return response.json()
            # 해당 오류가 발생시 무한 재시도 - 에러메세지 발생
            except requests.exceptions.ReadTimeout:
                logger.error(f"타임 아웃 재시도 {count} {url}")
            except requests.exceptions.ConnectionError:
                logger.error(f"요청 실패 재시도 {count} {url}")
            except requests.exceptions.HTTPError as e:
                logger.error(f"HTTPError 상태 코드 오류 \n"
                             f"{e}\n"
                             f"{type(e)}")
            # json이 아니라 xml로 오는 경우 dict로 인코딩을 하여 전달
            except JSONDecodeError as e:
                logger.error(f"JSONDecodeError 오류 - {e}")
                # xml 2 dict
                dict_type = xmltodict.parse(response.text)
                json_type = json.dumps(dict_type)
                dict2_type = json.loads(json_type)
                return dict2_type

            except Exception as e:
                # profiler.print_stats()
                logger.error(f"api 요청중 예상치 못한 오류 발생\n"
                             # f"{response.text}\n"
                             f"{response.headers}\n"
                             f"{e}\n"
                             f"{type(e)}")
            # 다시 api 호출시 대기 시간
            time.sleep(self.waiting_time)
